SUBROUTINE DECOMPOSITION

  USE GLOBAL, ONLY: PI2, V, RV, L2N, N, EPSILON, DEC, NITER, DT, TB,   &
                    FIND_KS, G_GAMA, INN_PROD, C_FUNC,                 &
                    GG, GS, FUNC_ALFA, TYPE_D
  USE nrtype
  USE nr

!----------------------------------------------------------------------

  IMPLICIT NONE

  CHARACTER (LEN=10) :: TLBUF

  LOGICAL      :: EXISTS

  INTEGER      :: I, J, IS, IP, IK, MIK, MIP, ILM, D1
  REAL    (DP) :: FI, NORMF2, MAXIPR, RES, S, P, K, L, M, KS, RI, T,   &
                  TP, TK, TL, TM, MAXIPO, SIP, RES1, TIME_BEGIN,       &
                  TIME_END, RVNORM, RVNORM1, RV_DECAY, LAMBDA1,        &
                  PERC_EN, COMP_TIMES, COMP_TIME
  COMPLEX (DP) :: MAXIP, KGAMAFI
  COMPLEX (DP), DIMENSION(:)  , ALLOCATABLE :: GGM, TRV
  REAL    (DP), DIMENSION(:,:), ALLOCATABLE :: GS_ALFA
  REAL    (SP), DIMENSION(:,:), ALLOCATABLE :: xi

	INTEGER(I4B), PARAMETER        :: NDIM=5
	REAL(SP)    , PARAMETER        :: FTOL=1.0e-6_sp
	INTEGER(I4B)                   :: iter
	REAL(SP)                       :: fret
	REAL(SP), DIMENSION(NDIM)      :: LMAXG

!----------------------------------------------------------------------

  OPEN  ( 20, FILE = 'BOOK.ORI', STATUS = 'NEW' )
  WRITE ( 20, '(A)' ) '   N  |<RnF,G_gama,fi>|   scale [s]   |  positio&
               &n [s] | frequency [Hz]|    Fi [rad]   |   K_gamma,fi  '
  WRITE ( 20, '(A)' ) ' -----+---------------+---------------+---------&
               &------+---------------+---------------+---------------'
  CLOSE ( 20 )

  ALLOCATE ( GS_ALFA(0:L2N+1,N), TRV(N), RV(N), GG(N), GGM(N),         &
                 DEC(N,0:7) )

!--- vypocet normy vstupneho signalu
  NORMF2 = 0._DP
  DO I = 1, N
    NORMF2 = NORMF2 + V(I)*V(I)
  END DO

!--- vypocet tabulkovych hodnot GS pre s z mnozony Gama_alfa
  DO J = 0, L2N
    IS = 2**J
    S  = REAL (IS, DP)
    KS = FIND_KS ( S )
    DO I = 1, N
      GS_ALFA(J,I) = GS( S, REAL(I,DP), KS )
    END DO
  END DO
  S  = REAL (2*N, DP)
  KS = FIND_KS ( S )
  DO I = 1, N
    GS_ALFA(J,I) = KS
  END DO

!--- NITER - aktualne cislo iteracie
!    RES   - reziduum
!    RV    - complexny nedecomponovany signal
  NITER = 0
  RES   = 1._DP
  RV    = CMPLX ( V    , 0._DP )

  IF     ( TYPE_D == 'o' ) THEN
    ALLOCATE ( xi(3,3) )
    D1 = 3
  ELSEIF ( TYPE_D == 'l' ) THEN
    ALLOCATE ( xi(4,4) )
    D1 = 4
  ELSEIF ( TYPE_D == 'q' ) THEN
    ALLOCATE ( xi(5,5) )
    D1 = 5
  ENDIF

    COMP_TIME = 0.

!!!!!!!! CYKLUS !!!!!!!!!!
  MAIN_CYCLE: DO
    print *,'NITER = ',NITER
!--- hladanie gama_n s vlnovkou, ktore dava maximalny sk. sucin 
!    <RV, G_gama> pre gama z Gama_alfa

!--- MAXIP - maximalny skalarny sucin
    MAXIP = CMPLX ( 0._DP, 0._DP )
    MAXIPR= 0.

    L = 0.
    M = 0.
    DO
      CALL CPU_TIME( TIME_BEGIN )
      MAXIPO = MAXIPR
      DO J = 0, L2N+1

        IF ( J == L2N+1 ) THEN
          IS = 2*N
        ELSE
          IS = 2**J
        END IF
        S      = REAL (IS, DP)
        MIP = 2*N/IS-1
        MIK = IS

        DO IP = 0, MIP

          P = S*REAL(IP,DP)/2._DP

          DO IK = 0, MIK

            K   = REAL(IK*N,DP)/S/2._DP

            SIP = FUNC_ALFA (GS_ALFA(J,1:N), P, K, L, M)
            IF ( ABS(SIP) > ABS(MAXIPR) ) THEN
              MAXIPR    = SIP
              LMAXG (1) = REAL(S, SP)
              LMAXG (2) = REAL(P, SP)
              LMAXG (3) = REAL(K, SP)
              LMAXG (4) = REAL(L, SP)
              LMAXG (5) = REAL(M, SP)
            END IF

          END DO

        END DO

      END DO

  ! hladanie gama_n z mnoziny Gama
      IF     ( TYPE_D == 'o' ) THEN
        xi = reshape( (/ 1000.0_sp, 1000.0_sp, 1000.0_sp,          &
                         1000.0_sp,-1000.0_sp, 1000.0_sp,          &
                         1000.0_sp, 1000.0_sp,-1000.0_sp /),       &
                      (/ 3, 3 /) ) 
      ELSEIF ( TYPE_D == 'l' ) THEN
        xi = reshape( (/ 1000.0_sp, 1000.0_sp, 1000.0_sp, 0.0_sp,  &
                         1000.0_sp,-1000.0_sp, 1000.0_sp, 0.0_sp,  &
                         1000.0_sp, 1000.0_sp,-1000.0_sp, 0.0_sp,  &
                         0.0_sp, 0.0_sp, 0.0_sp, 1.0_sp /),        &
                      (/ 4, 4 /) )
      ELSEIF ( TYPE_D == 'q' ) THEN
        xi = reshape( (/ 1000.0_sp, 1000.0_sp, 1000.0_sp, 0.0_sp, 0.0_sp,  &
                         1000.0_sp,-1000.0_sp, 1000.0_sp, 0.0_sp, 0.0_sp,  &
                         1000.0_sp, 1000.0_sp,-1000.0_sp, 0.0_sp, 0.0_sp,  &
                         0.0_sp, 0.0_sp, 0.0_sp, 1.0_sp, 0.0_sp,           &
                         0.0_sp, 0.0_sp, 0.0_sp, 0.0_sp, 1.0_sp /),        &
                      (/ NDIM,NDIM /) )
      ENDIF

      CALL POWELL (LMAXG(1:D1),xi,FTOL,iter,fret)
      IF ( NINT(LMAXG(1)) >= 2*N ) LMAXG(1) = REAL(2*N,SP)
      MAXIP = C_FUNC (LMAXG)

! urcenie Fi ako komplexnej fazy skalarneho sucinu <Rnf,g_gama>
      FI = ATAN2 ( IMAG(MAXIP), REAL(MAXIP) )

      GGM= CONJG  ( GG )
      KGAMAFI = SQRT(   2._DP /                                          &
                      ( 1._DP + REAL ( EXP ( CMPLX (0._DP, 2._DP*FI) )   &
                              * INN_PROD (GG, GGM), DP) ) )

  ! vypocet <RnF,g_gama_fi>
      MAXIPR = KGAMAFI*REAL ( MAXIP * EXP ( CMPLX(0._DP,-FI) ), DP)

      IF ( (ABS(MAXIPR-MAXIPO))/ABS(MAXIPR) < 0.00001 ) EXIT 
    END DO

! vypocet K(gama,Fi)
    S  = REAL ( LMAXG(1), DP )
    P  = REAL ( LMAXG(2), DP )
    K  = REAL ( LMAXG(3), DP )
    L  = REAL ( LMAXG(4), DP )
    M  = REAL ( LMAXG(5), DP )
    KS = FIND_KS ( S )

    NITER = NITER + 1
    DEC (NITER, 0) = MAXIPR
    DEC (NITER, 1) = S
    DEC (NITER, 2) = P
    DEC (NITER, 3) = K
    DEC (NITER, 4) = FI
    DEC (NITER, 5) = KGAMAFI
    DEC (NITER, 6) = L
    DEC (NITER, 7) = M
    
    RVNORM  = 0.
    DO I=1,N
      RVNORM  = RVNORM  + RV(I)*RV(I)
    END DO
! vypocet rezidua
    TRV = MAXIPR*KGAMAFI*( GG *EXP ( CMPLX(0._DP, FI) )                &
                         + GGM*EXP ( CMPLX(0._DP,-FI) ) )/2._DP
    RV  = RV  - TRV

    RVNORM1  = 0.
    DO I=1,N
      RVNORM1  = RVNORM1  + RV(I)*RV(I)
    END DO

    RES  =  RES - MAXIPR*MAXIPR/NORMF2
    RES1 = SQRT( RVNORM1/NORMF2 )
!    print *,'RESIDUUM  = ',res
    print *,'RESIDUUM  = ',res1
    
    WRITE ( TLBUF, '(A3, I3.3, A4)' )  'VEC', NITER, '.DAT'
    OPEN  ( 20, FILE = TLBUF(1:10), STATUS = 'NEW' )
    DO I = 1, N
      WRITE ( 20, *) REAL(I-1,SP)*DT+TB, REAL(TRV(I),SP)
    END DO
    CLOSE ( 20 )
    
    OPEN  ( 20, FILE = 'BOOK.ORI', STATUS = 'OLD',                     &
                POSITION = 'APPEND' )
    TP = DEC(NITER,2)*DT
    TK = DEC(NITER,3)/DT/REAL(N,DP)
    TL = DEC(NITER,6)/DT/DT/REAL(N,DP)
    TM = DEC(NITER,7)/DT/DT/DT/REAL(N,DP)
    WRITE ( 20, '(1X, I5, 8(1X, G15.6))' ) NITER,                      &
                                           DEC(NITER,0),               &
                                           DEC(NITER,1)*DT,            &
                                           TP+TB,                      &
                                           TK+TL*TP,                   &
                                           DEC(NITER,4),               &
                                           DEC(NITER,5),               &
                                           TL+TM*TP,                   &
                                           3.*TM
                                           
    CLOSE ( 20 )
    
    INQUIRE (FILE = 'STOP', EXIST = EXISTS)

    IF ( ( RES1 <= EPSILON ) .OR. EXISTS .OR. NITER > N ) EXIT MAIN_CYCLE


  END DO MAIN_CYCLE

  OPEN  ( 20, FILE = 'VECRES.DAT', STATUS = 'NEW' )
  DO I = 1, N
    WRITE ( 20, *) REAL(I-1,SP)*DT+TB, REAL(RV(I),SP)
  END DO
  CLOSE ( 20 )

  
END SUBROUTINE DECOMPOSITION