SUBROUTINE CWT

  USE GLOBAL, ONLY: WP,WVLT_TYPE, W0, M, DT, N, PI, OFMAX,        &
                    FMAX, FMIN, PCOI, KEY_LOG, KNFFT, NFFT, NFFT21,    &    
                    F_V, FWV, WV, FPWV, FMINLOG, FMAXLOG

  IMPLICIT NONE
!----------------------------------------------------------------------
  
  INTEGER  :: I,J, JJ
  REAL(WP) :: S, F, DF, DFA, COI

!----------------------------------------------------------------------

  INTERFACE 

    FUNCTION     F_W_1      ( S, FA )
      USE GLOBAL, ONLY: WP
      REAL(WP), INTENT(IN) :: S, FA
      COMPLEX(WP)          :: F_W_1
    END FUNCTION F_W_1   
!   -------------
    FUNCTION     F_W_2      ( S, FA )
      USE GLOBAL, ONLY: WP
      REAL(WP), INTENT(IN) :: S, FA
      COMPLEX(WP)          :: F_W_2
    END FUNCTION F_W_2   
!   -------------
    FUNCTION     F_W_3      ( S, FA )
      USE GLOBAL, ONLY: WP
      REAL(WP), INTENT(IN) :: S, FA
      COMPLEX(WP)          :: F_W_3
    END FUNCTION F_W_3
  END INTERFACE

!----------------------------------------------------------------------

! F-FREKVENCIA PRE KTORU SA POCITA CASOVO-FREKVENCNA ANALYZA
! (J-1)*DFA - "PRACOVNA" FREKVENCIA PRE VYPOCET KONVOLUCIE VO FOUR.OBL.

  DF  = 1./(N    *DT)
  DFA = 1./(NFFT*DT)

  IF (KEY_LOG) THEN
    FMAXLOG = LOG10 (FMAX)
    FMINLOG = LOG10 (FMIN)
  ENDIF

  DO I = 1, OFMAX
    IF (KEY_LOG) THEN
      F = 10**( REAL(I-1,WP)/REAL(OFMAX-1,WP)*(FMAXLOG-FMINLOG) + FMINLOG)
    ELSEIF (WVLT_TYPE == 1 ) THEN
      F =       REAL(I-1,WP)/REAL(OFMAX-1,WP)*(FMAX   -FMIN   ) + FMIN
    ELSE
      F =       REAL(I  ,WP)/REAL(OFMAX-1,WP)*(FMAX   -FMIN   ) + FMIN        
    ENDIF
    
    SELECT CASE ( WVLT_TYPE )

      CASE (1)
        S = W0 / (2.*PI*F)   
        DO J = 1, NFFT21
          FPWV(J) = F_W_1( S,REAL((J-1),WP)*DFA )
        END DO

      CASE (2)
        S = (2.*REAL(M)+1.)      / (4.*PI*F)
        DO J = 1, NFFT21
          FPWV(J) = F_W_2( S,REAL((J-1),WP)*DFA )
        END DO

      CASE (3)
        S = SQRT(REAL(M)+0.5)    / (2.*PI*F)
        DO J = 1, NFFT21
          FPWV(J) = F_W_3( S,REAL((J-1),WP)*DFA )
        END DO

    END SELECT
    
    DO  J = NFFT21+1, NFFT
      IF ( WVLT_TYPE < 3 ) THEN
        FPWV(J)     = (0._WP,0._WP)
      ELSE
        FPWV(J)     = CONJG(FPWV(NFFT-J+2))
      END IF
    END DO
    
    DO J = 1, NFFT
      FWV (J) = F_V (J)*CONJG( FPWV(J) )
      FPWV(J) = FPWV(J)*CONJG( FPWV(J) )
    END DO
    
    CALL FCOOLR (KNFFT,FWV ,1._WP)
    CALL FCOOLR (KNFFT,FPWV,1._WP)              

    WV(1:N,I) = FWV(1:N)*DFA 
    COI = MAXVAL ( ABS (FPWV(1:N)) ) * PCOI     
    JJ = 1                                     
    DO J = 1, N
      IF ( ABS(FPWV(J)) > COI ) JJ = J
    END DO
    WV(N+1,I) = CMPLX(JJ,0.)                     

  END DO
END SUBROUTINE CWT