SUBROUTINE READ_INP_PARAM

  USE GLOBAL, ONLY: INP_FILE, OUTf_FILE, TB, TE, DT, FMIN, FMAX, WTW,  &
                    TW_TYPE,                                           &
                    OUTo_FILE, OUTr_FILE, OUTt_FILE, OFMAX, KEY_LOG,   &
                    K1, K2, K3, K4, KNFFT, NFFT, NFFT21, N , PERC, WP, &
                    FMAXLOG, FMINLOG, DF_LOG 

  IMPLICIT NONE
  
  INTEGER :: KNFFT_N, NT, NF
  REAL(WP):: DF1

  NAMELIST /CONTROL_DATA/ INP_FILE, TB, TE, DT, FMAX,                  &
                          WTW, TW_TYPE, NF, FMIN, K1, K2, K3, K4,      &
                          KEY_LOG, KNFFT, NT
  NAMELIST /TYPE_1/   PERC

!----------------------------------------------------------------------

  NT      = -1
  PERC    = 0.1
  TB      = -9999.
  TE      = -9999.
  KNFFT   = 15
  FMIN    = -1.
  FMAX    = -1.
  NF      = 500
  WTW     = 1.0

  K1 = .TRUE.
  K2 = .FALSE.
  K3 = .FALSE.
  K4 = .TRUE.
 
  KEY_LOG =.FALSE.
  

  OPEN ( 10, FILE="HFRWFT", STATUS="OLD" )
  READ ( 10, NML=CONTROL_DATA )
  
  SELECT CASE ( TW_TYPE )
    CASE (1)
      READ ( 10, NML=TYPE_1 )
  END SELECT 

  IF  (( K1.OR.K2.OR.K3) .AND. KEY_LOG ) THEN
    print *, ' This program computes reassignment method only in linear frequency scale.'
    print *, ' WFT without reassignment method will be computed.'
    K1 = .FALSE.
    K2 = .FALSE.
    K3 = .FALSE.
  ENDIF  

  IF ( NT < 0 ) THEN
    print *, ' The value of parameter "NT" (number of samples) '
    print *, ' in the file "HFRWFT" is missing !!! '
    STOP
  ENDIF
  IF ( TB < -9000. ) THEN
    TB = 0.
    TE = (NT-1)*DT
  END IF

  IF ( TE < -9000. ) THEN
    TE = TB + (NT-1)*DT
  END IF

  IF ( NINT((TE-TB)/DT)+1 /= NT ) THEN
    print *, ' TE has to be equal to TB + (NT-1)*DT !!! '
    STOP
  END IF

  IF ( FMAX < 0. ) THEN
    FMAX = 0.5/DT
  END IF

  IF ( FMIN < 0. ) THEN
    IF ( KEY_LOG ) THEN
      FMIN = FMAX/1000.
    ELSE
      FMIN = 0.
    END IF
  END IF

  IF (KEY_LOG) THEN
    FMAXLOG = LOG10 (FMAX)
    FMINLOG = LOG10 (FMIN)
    DF_LOG  = (FMAXLOG-FMINLOG)/REAL(NF-1,WP)
    DF1 = 10**( DF_LOG + FMINLOG ) - FMIN
    KNFFT_N = INT(LOG(1./DF1/DT)/LOG(2.))+1
    IF ( KNFFT_N > 19 ) THEN
      print *, ' TOO MANY FREQUENCY SAMPLES !!! '
      STOP
    END IF
    IF ( KNFFT < KNFFT_N ) KNFFT = KNFFT_N
  END IF

  NFFT   = 2**KNFFT
  NFFT21 = NFFT/2 + 1

  CLOSE ( 10 )

  OUTo_FILE = INP_FILE (1:LEN_TRIM(INP_FILE))//'o.GRD'
  OUTt_FILE = INP_FILE (1:LEN_TRIM(INP_FILE))//'t.GRD'
  OUTf_FILE = INP_FILE (1:LEN_TRIM(INP_FILE))//'f.GRD'
  OUTr_FILE = INP_FILE (1:LEN_TRIM(INP_FILE))//'r.GRD'
  INP_FILE  = INP_FILE (1:LEN_TRIM(INP_FILE))//'.DAT'

  N     = NT
  OFMAX = NF
  
END SUBROUTINE READ_INP_PARAM
      