PROGRAM TFPLOT

USE PRECISION, ONLY: WP
USE DFLIB

!----------------------------------------------------------------------

IMPLICIT NONE

  CHARACTER (LEN= 1) :: KEY, KEY1, KEY2, P
  CHARACTER (LEN= 3) :: TFA, WVL2, MPD_TYPE
  CHARACTER (LEN=15) :: SAVEBMP, INCH, INP_FILE
  CHARACTER (LEN=12) :: DES_FILE
  CHARACTER (LEN=17) :: WVL1
  CHARACTER (LEN=80) :: OUTCH, OUTCH1, COMMENT

  LOGICAL    :: KEY_SQ, KEY_SC, KEY_MAX, KEY_GRID, KEY_LOG, KEY_VIEW,  &
                KEY_INTER, EXISTS, KEY_SAVE, TICS_F, TICS_T, KEY_SUMT, &
                KEY_AMPLOG
  INTEGER(4) :: RES

  INTEGER    :: N, NDDT, NDT, NDDF, NDF, NUM_COLS, W_TYPE, L_W,        &
                WVLT_TYPE, ORD, OFMAX, MTMIN, MTMAX, MFMIN, MFMAX,     &
                LINCH, I, J, CC, DICT

  REAL(WP)   :: WW, MAG, CG, MAXSC, FMIN, FMAX, TMIN, TMAX, MFHZ, DT,  &
                TB, TE, FB, FE, W0, TMINTIC, FMINTIC, DTTICS, DFTICS,  &
                TMIN1, TMAX1, FMIN1, FMAX1, GLMAX, O, PERC


  REAL(WP), DIMENSION(:,:), ALLOCATABLE :: U
  REAL(WP), DIMENSION(:)  , ALLOCATABLE :: DAT

  TYPE(WINDOWCONFIG):: WC     
  TYPE(QWINFO)      :: WINFO
  TYPE(RCCOORD)     :: RC

!----------------------------------------------------------------------

  INTERFACE
    SUBROUTINE TSFVIEW ( U, DAT, KEY_SQ, KEY_INTER,                    &
                         FMIN, FMAX, TMIN, TMAX,                       &
                         MTMIN, MTMAX, MFMIN, MFMAX,                   &
                         MAG, MAXSC, KEY_LOG, KEY_GRID, KEY_MAX, WW,   &
                         KEY_SUMT,                                     &
                         DES_FILE, NUM_COLS, CG, SAVEBMP,              &
                         TMINTIC, DTTICS, FMINTIC, DFTICS,             &
                         NDT, NDDT, NDF, NDDF )
      USE PRECISION, ONLY: WP
      IMPLICIT NONE
      CHARACTER (LEN=*) , INTENT(IN)  :: DES_FILE, SAVEBMP

      LOGICAL,            INTENT(IN)  :: KEY_SQ, KEY_GRID, KEY_MAX,    &
                                         KEY_LOG, KEY_INTER, KEY_SUMT
      INTEGER,            INTENT(IN)  :: MTMIN, MTMAX, MFMIN, MFMAX,   &
                                         NDT, NDDT, NDF, NDDF,         &
                                         NUM_COLS
      REAL(WP),           INTENT(IN)  :: FMIN, FMAX, TMIN, TMAX, MAG,  &
                                         TMINTIC, FMINTIC,             &
                                         DTTICS, DFTICS,               &
                                         CG, MAXSC, WW

      REAL(WP), DIMENSION(:)  , INTENT(INOUT) :: DAT
      REAL(WP), DIMENSION(:,:), INTENT(INOUT) :: U
    END SUBROUTINE TSFVIEW     !-------------------------------------

    SUBROUTINE TEST_BORDERS ( P, LMIN, LMAX, GMIN, GMAX, KEY_LOG )

      USE PRECISION, ONLY: WP
      IMPLICIT NONE
      CHARACTER (1),     INTENT(IN)     :: P
      LOGICAL,           INTENT(IN)     :: KEY_LOG
      REAL (WP),         INTENT(IN)     :: GMIN, GMAX
      REAL (WP),         INTENT(INOUT)  :: LMIN, LMAX

    END SUBROUTINE TEST_BORDERS  !------------------------------------

    SUBROUTINE SETTINGS( COMMENT, KEY_VIEW, KEY_INTER, KEY_MAX, MAG,    &
                         KEY_GRID, KEY_SAVE, DES_FILE, MAXSC, GLMAX,   &
                         NDT, NDDT, TMINTIC, DTTICS, TMIN, TMAX,       &
                         NDF, NDDF, FMINTIC, DFTICS, FMIN, FMAX, KEY_LOG)

      USE PRECISION, ONLY: WP
      USE DFLIB
      IMPLICIT NONE
      CHARACTER (LEN=*) , INTENT(IN)  :: DES_FILE, COMMENT
      LOGICAL,            INTENT(IN)  :: KEY_VIEW, KEY_GRID, KEY_MAX,  &
                                         KEY_INTER, KEY_SAVE, KEY_LOG
      INTEGER,            INTENT(IN)  :: NDT, NDDT, NDF, NDDF
      REAL(WP),           INTENT(IN)  :: MAG, MAXSC,GLMAX,             &
                                         TMINTIC, FMINTIC, TMIN, TMAX, &
                                         DTTICS, DFTICS, FMIN, FMAX
    END SUBROUTINE SETTINGS   ! ------------------------------------------

    SUBROUTINE AXIS ( AMAX, DTICS, NDD, ND )

      USE PRECISION, ONLY: WP
      IMPLICIT NONE
      REAL (WP),         INTENT(IN   )  :: AMAX
      INTEGER,           INTENT(INOUT)  :: NDD, ND
      REAL (WP),         INTENT(INOUT)  :: DTICS
    END SUBROUTINE AXIS   !-----------------------------------------------

  END INTERFACE

!----------------------------------------------------------------------

  NAMELIST /INPUT/        INP_FILE

  NAMELIST /CONTROL_DATA/ FMIN, FMAX, TMIN, TMAX,                      &
                          KEY_SQ, KEY_SC, KEY_MAX, KEY_AMPLOG,         &
                          NDDT, NDT, NDDF, NDF,                        &
                          KEY_GRID, CG, MAG, DES_FILE,                 &
                          TMINTIC, DTTICS, FMINTIC, DFTICS, KEY_SUMT

!----------------------------------------------------------------------

  OPEN  ( 10, FILE='HFTFPLOT', STATUS='OLD' )
  READ  ( 10, NML = INPUT ) 

!------- read TFA parameters from .GRD file ------------

  OPEN ( 18, FILE = INP_FILE, FORM   ='UNFORMATTED', STATUS = 'OLD')

  READ ( 18 )   TFA
  
  IF (TFA .EQ. 'CWT') THEN

    READ ( 18 )   WVLT_TYPE

    SELECT CASE ( WVLT_TYPE )
      CASE (1)
        READ ( 18 ) W0
        O = W0
        WVL1 = 'Morlet, omega0 = '
      CASE (2)
        READ ( 18 ) ORD
        O = ORD
        WVL1 = 'Paul, order = '
      CASE (3)
        READ ( 18 ) ORD
        O = ORD
        WVL1 = 'DOG, order = '
    END SELECT
   
  ELSEIF (TFA .EQ. 'WFT') THEN    
    
    READ ( 18 )  WW
    READ ( 18 )  W_TYPE
    READ ( 18 )  PERC
    WVL1  = 's, cos-taper, ' 
    WVL2  = '%'
          
  ELSEIF (TFA .EQ. 'MPD') THEN    

    READ ( 18 )  DICT
    IF ( DICT == 1 ) MPD_TYPE = 'ori'
    IF ( DICT == 2 ) MPD_TYPE = 'lin'
    IF ( DICT == 3 ) MPD_TYPE = 'qua'
    WVL2 = ' '
  ENDIF

  READ ( 18 ) N, DT, TB, TE, FB, FE, OFMAX, KEY_LOG  

  IF ( KEY_LOG ) THEN
    MFHZ= REAL (OFMAX-1,WP) / LOG10(FE/FB)
  ELSE
    MFHZ= REAL (OFMAX-1,WP) /      (FE-FB)
  END IF

  ALLOCATE ( U(N+1, OFMAX), DAT(N) )

!------- read control parameters -------

  KEY_SQ  = .FALSE.
  KEY_SC  = .FALSE.
  KEY_MAX = .FALSE.
  KEY_GRID= .FALSE.
  KEY_INTER=.TRUE.
  KEY_SAVE =.TRUE.
  MAG     = 0.64
  NDDT    = 0
  NDT     = 5
  NDDF    = 1
  NDF     = 5  
  FMIN    = FB
  FMAX    = FE
  TMIN    = TB
  TMAX    = TE
  CG      = 0.88
  DES_FILE = 'SCALE.DEcoi4'
  SAVEBMP  = 'OUT.BMP'
  TMINTIC  = TMIN
  DTTICS   = -9999.
  FMINTIC  = FMIN
  DFTICS   = -9999.
  COMMENT = '-'
  KEY_VIEW = .TRUE.
  KEY_SUMT = .FALSE.
  KEY_AMPLOG =.FALSE.

  READ ( 10, NML=CONTROL_DATA )

  MAXSC = -1.
  IF ( KEY_SC ) READ (10, *) MAXSC

  CLOSE( 10 )

  IF (TFA /= 'WFT')  WW = -1.
  
  IF (KEY_LOG .AND. FMIN==0.) THEN
     FMIN = FMAX / 1000.
  ENDIF

!------- colors

  OPEN ( 10, FILE = DES_FILE )

  READ ( 10,* ) NUM_COLS

  CLOSE( 10 )
! ------------ definition of windows ----------------------------------
                           ! frame window
  WINFO.H    = 775
  WINFO.W    = 1145
  WINFO.X    = 0
  WINFO.Y    = 0
  WINFO.TYPE = QWIN$SET
  RES        = SETWSIZEQQ(QWIN$FRAMEWINDOW,WINFO)

  RES = PASSDIRKEYSQQ(.TRUE.)

!----------------------------------------------------------------------
  WINFO.H    = 3          !  dialog ('graphic') window
  WINFO.W    = 88
  WINFO.X    = 0
  WINFO.Y    = 44
  WINFO.TYPE = QWIN$SET
  RES        = SETWSIZEQQ(0,WINFO)
  wc.numxpixels  = -1
  wc.numypixels  = -1
  wc.numtextcols = 88
  wc.numtextrows = 3
  wc.numcolors   = -1
  wc.title       = "DIALOG WINDOW"C
  wc.fontsize    = QWIN$EXTENDFONT
  wc.extendfontname = "Fixedsys"C
  wc.extendfontsize    = #00080010
  wc.extendfontattributes = QWIN$EXTENDFONT_NORMAL
  RES = SETWINDOWCONFIG(WC)
  IF ( .NOT.(RES) ) RES = SETWINDOWCONFIG(WC)
  RES = SETTEXTCOLORRGB(#00C0C0C0)
!--------------------------------------------------------------------
  OPEN ( 19, FILE='USER', TITLE='ACTUAL SETTINGS OF DISPLAY' )
  WINFO.H    = 10
  WINFO.W    = 88
  WINFO.X    = 0
  WINFO.Y    = 34
  WINFO.TYPE = QWIN$SET
  RES        = SETWSIZEQQ(19,WINFO)

  wc.numxpixels  = -1
  wc.numypixels  = -1
  wc.numtextcols = 88
  wc.numtextrows = 10
  wc.numcolors   = -1
  wc.title       = "ACTUAL SETTINGS OF DISPLAY"C
  wc.fontsize    = QWIN$EXTENDFONT
  wc.extendfontname = "Fixedsys"C
  wc.extendfontsize    = #00080010
  wc.extendfontattributes = QWIN$EXTENDFONT_NORMAL
  RES = SETWINDOWCONFIG(WC)
  IF ( .NOT.(RES) ) RES = SETWINDOWCONFIG(WC)
  RES = SETTEXTCOLORRGB(#00C0C0C0)

!------------------------------------------------------------------
  OPEN ( 20, FILE='USER', TITLE='HELP' )    ! help window
  WINFO.H    = 47
  WINFO.W    = 54
  WINFO.X    = 89
  WINFO.Y    = 0
  WINFO.TYPE = QWIN$SET
  RES        = SETWSIZEQQ(20,WINFO)

  wc.numxpixels  = -1
  wc.numypixels  = -1
  wc.numtextcols = 54
  wc.numtextrows = 47
  wc.numcolors   = -1
  wc.title       = "HELP"C
  wc.fontsize    = QWIN$EXTENDFONT
  wc.extendfontname = "Fixedsys"C
  wc.extendfontsize    = #00080010
  wc.extendfontattributes = QWIN$EXTENDFONT_NORMAL
  RES = SETWINDOWCONFIG(WC)
  IF ( .NOT.(RES) ) RES = SETWINDOWCONFIG(WC)
  RES = SETTEXTCOLORRGB(#0000FFFF)                 ! yellow

  WRITE(20,*) 'TIME-FREQUENCY ANALYSIS:'
  WRITE(20,*) '========================'
!  Write(20,'(A)') TFA
  IF (TFA .EQ. 'CWT') THEN
    L_W=LEN_TRIM (WVL1)

    WRITE(20,'(A\)') ' CWT, wavelet used: '
    RES = SETTEXTCOLORRGB(#00FFFFFF)                 ! white (bright)
    SELECT CASE ( WVLT_TYPE )
      CASE (1)
        WRITE(20,'(A,F2.0)')  WVL1, W0
      CASE (2)
        WRITE(20,'(A,F3.0)')  WVL1(1:L_W), O
      CASE (3)
        WRITE(20,'(A,F3.0)')  WVL1(1:L_W), O
    END SELECT
 
  ELSEIF (TFA .EQ. 'WFT') THEN    
    WRITE(20,'(A\)') ' WFT, window used: '
    RES = SETTEXTCOLORRGB(#00FFFFFF)                 ! white (bright)
    WRITE(20,'(F4.0, A, F3.0, A)') WW, WVL1(1:14), 100.*PERC, WVL2
        
  ELSEIF (TFA .EQ. 'MPD') THEN    
    WRITE(20,'(A\)') ' MPD, Gabor dictionary: '
    RES = SETTEXTCOLORRGB(#00FFFFFF)                 ! white (bright)
    WRITE(20,'(A, A)') WVL2, MPD_TYPE
  ENDIF
  
  RES = SETTEXTCOLORRGB(#0000FFFF)                 ! yellow
  WRITE(20,'(A\)') ' Borders of calculated TF plane (file: '
  RES = SETTEXTCOLORRGB(#00FFFFFF)                 ! white (bright)
  WRITE(20,'(A12\)')  INP_FILE
  RES = SETTEXTCOLORRGB(#0000FFFF)                 ! yellow
  WRITE(20,*) '):'
  WRITE(20,'(A\)') '   TB = '
  RES = SETTEXTCOLORRGB(#00FFFFFF)                 ! white (bright)
  WRITE(20,'(F10.3,A\)') TB,' s   '
  RES = SETTEXTCOLORRGB(#0000FFFF)                 ! yellow
  WRITE(20,'(A\)') '   FB = '
  RES = SETTEXTCOLORRGB(#00FFFFFF)                 ! white (bright)
  WRITE(20,'(F5.1,A)') FB,' Hz'
  RES = SETTEXTCOLORRGB(#0000FFFF)                 ! yellow
  WRITE(20,'(A\)') '   TE = '
  RES = SETTEXTCOLORRGB(#00FFFFFF)                 ! white (bright)
  WRITE(20,'(F10.3,A\)') TE,' s   '
  RES = SETTEXTCOLORRGB(#0000FFFF)                 ! yellow
  WRITE(20,'(A\)') '   FE = '
  RES = SETTEXTCOLORRGB(#00FFFFFF)                 ! white (bright)
  WRITE(20,'(F5.1,A)') FE,' Hz'
  RES = SETTEXTCOLORRGB(#00C8FFFF)                 ! light-yellow
  WRITE(20,*) ' '
  WRITE(20,*) 'MAIN MENU:'
  WRITE(20,*) '---------- '
!----------------- v ----------------
  RES = SETTEXTCOLORRGB(#0091C8FF)                 ! light-orange;      
  WRITE(20,'(A\)') ' v'
  RES = SETTEXTCOLORRGB(#00C0C0C0)                 ! white
  WRITE(20,*) '.... (v)iew actual, changed TFA plot or not'
  WRITE(20,*) 
!----------------- t ----------------
  RES = SETTEXTCOLORRGB(#0091C8FF)                 ! light-orange
  WRITE(20,'(A\)') ' t'
  RES = SETTEXTCOLORRGB(#00C0C0C0)                 ! white
  WRITE(20,*) '.... set plotting parameters in (t)ime direction'
  RES = SETTEXTCOLORRGB(#00FFFFFF)                 ! white (bright)
  WRITE(20,'(A\)') '       b'
  RES = SETTEXTCOLORRGB(#00C0C0C0)                 ! white
  WRITE(20,*) '... set new (b)orders of TF plane in time'
  RES = SETTEXTCOLORRGB(#00FFFFFF)                 ! white (bright)
  WRITE(20,'(A\)') '       a'
  RES = SETTEXTCOLORRGB(#00C0C0C0)                 ! white
  WRITE(20,*) '... set new (a)xis parameters: tics, labels'
  WRITE(20,*) 
!----------------- f ----------------
  RES = SETTEXTCOLORRGB(#0091C8FF)                 ! svetlo-oranzova
  WRITE(20,'(A\)') ' f'
  RES = SETTEXTCOLORRGB(#00C0C0C0)                 ! biela (tlmena)
  WRITE(20,*) '.... set plotting parameters in (f)req. direction'
  RES = SETTEXTCOLORRGB(#00FFFFFF)                 ! biela (jasna)
  WRITE(20,'(A\)') '       b'
  RES = SETTEXTCOLORRGB(#00C0C0C0)                 ! biela (tlmena)
  WRITE(20,*) '... set new (b)orders of TF plane in freq.'
  RES = SETTEXTCOLORRGB(#00FFFFFF)                 ! biela (jasna)
  WRITE(20,'(A\)') '       a'
  RES = SETTEXTCOLORRGB(#00C0C0C0)                 ! biela (tlmena)
  WRITE(20,*) '... set new (a)xis parameters: tics, labels'
  WRITE(20,*) 
!------------------ g ---------------
  RES = SETTEXTCOLORRGB(#0091C8FF)                 ! svetlo-oranzova
  WRITE(20,'(A\)') ' g'
  RES = SETTEXTCOLORRGB(#00C0C0C0)                 ! biela (tlmena)
  WRITE(20,*) '.... set (g)rid plotting'
  RES = SETTEXTCOLORRGB(#00FFFFFF)                 ! biela (jasna)
  WRITE(20,'(A\)') '       d'
  RES = SETTEXTCOLORRGB(#00C0C0C0)                 ! biela (tlmena)
  WRITE(20,*) '... (d)raw grid or not'
  RES = SETTEXTCOLORRGB(#00FFFFFF)                 ! biela (jasna)
  WRITE(20,'(A\)') '       c'
  RES = SETTEXTCOLORRGB(#00C0C0C0)                 ! biela (tlmena)
  WRITE(20,*) '... set new grid (c)olor'
  WRITE(20,*) 
!------------------ s ---------------
  RES = SETTEXTCOLORRGB(#0091C8FF)                 ! svetlo-oranzova
  WRITE(20,'(A\)') ' s'
  RES = SETTEXTCOLORRGB(#00C0C0C0)                 ! biela (tlmena)
  WRITE(20,*) '.... set color (s)cale parameters'
  RES = SETTEXTCOLORRGB(#00FFFFFF)                 ! biela (jasna)
  WRITE(20,'(A\)') '       m'
  RES = SETTEXTCOLORRGB(#00C0C0C0)                 ! biela (tlmena)
  WRITE(20,*) '... set new value for (m)aximum on scale'
  RES = SETTEXTCOLORRGB(#00FFFFFF)                 ! biela (jasna)
  WRITE(20,'(A\)') '       f'
  RES = SETTEXTCOLORRGB(#00C0C0C0)                 ! biela (tlmena)
  WRITE(20,*) '... set (f)ile with new scale description'
  WRITE(20,*) 
!------------------ m ---------------
  RES = SETTEXTCOLORRGB(#0091C8FF)                 ! svetlo-oranzova
  WRITE(20,'(A\)') ' m'
  RES = SETTEXTCOLORRGB(#00C0C0C0)                 ! biela (tlmena)
  WRITE(20,*) '.... set (m)agnification of image'
  WRITE(20,*) 
! ------------------ r --------------
  RES = SETTEXTCOLORRGB(#0091C8FF)                 ! svetlo-oranzova
  WRITE(20,'(A\)') ' r'
  RES = SETTEXTCOLORRGB(#00C0C0C0)                 ! biela (tlmena)
  WRITE(20,*) '.... plot (r)idges of local maxima or not'
  WRITE(20,*) 
! ------------------ z --------------
  RES = SETTEXTCOLORRGB(#0091C8FF)                 ! svetlo-oranzova
  WRITE(20,'(A\)') ' z'
  RES = SETTEXTCOLORRGB(#00C0C0C0)                 ! biela (tlmena)
  WRITE(20,*) '.... set (z)oom - by factor of 2'
  RES = SETTEXTCOLORRGB(#00FFFFFF)                 ! biela (jasna)
  WRITE(20,'(A\)') '       f'
  RES = SETTEXTCOLORRGB(#00C0C0C0)                 ! biela (tlmena)
  WRITE(20,*) '... in (f)requency direction'
  RES = SETTEXTCOLORRGB(#00FFFFFF)                 ! biela (jasna)
  WRITE(20,'(A\)') '            i'
  RES = SETTEXTCOLORRGB(#00C0C0C0)                 ! biela (tlmena)
  WRITE(20,*) '... zoom (i)n'
  RES = SETTEXTCOLORRGB(#00FFFFFF)                 ! biela (jasna)
  WRITE(20,'(A\)') '            o'
  RES = SETTEXTCOLORRGB(#00C0C0C0)                 ! biela (tlmena)
  WRITE(20,*) '... zoom (o)ut'
  RES = SETTEXTCOLORRGB(#00FFFFFF)                 ! biela (jasna)
  WRITE(20,'(A\)') '       t'
  RES = SETTEXTCOLORRGB(#00C0C0C0)                 ! biela (tlmena)
  WRITE(20,*) '... in (t)ime direction'
  RES = SETTEXTCOLORRGB(#00FFFFFF)                 ! biela (jasna)
  WRITE(20,'(A\)') '            i'
  RES = SETTEXTCOLORRGB(#00C0C0C0)                 ! biela (tlmena)
  WRITE(20,*) '... zoom (i)n'
  RES = SETTEXTCOLORRGB(#00FFFFFF)                 ! biela (jasna)
  WRITE(20,'(A\)') '            o'
  RES = SETTEXTCOLORRGB(#00C0C0C0)                 ! biela (tlmena)
  WRITE(20,*) '... zoom (o)ut'
  WRITE(20,*) 
! ------------------ i --------------
  RES = SETTEXTCOLORRGB(#0091C8FF)                 ! svetlo-oranzova
  WRITE(20,'(A\)') ' i'
  RES = SETTEXTCOLORRGB(#00C0C0C0)                 ! biela (tlmena)
  WRITE(20,*) '.... plot (i)nterpolated TF plane or not'
  WRITE(20,*) 
  RES = SETTEXTCOLORRGB(#0091C8FF)                 ! svetlo-oranzova
  WRITE(20,'(A\)') ' c'
  RES = SETTEXTCOLORRGB(#00C0C0C0)                 ! biela (tlmena)
  WRITE(20,*) '.... set (c)omment - max. 80 characters'
  WRITE(20,*) 

! --------------ARROWS--------------
  RES = SETTEXTCOLORRGB(#00C8FFFF)                 ! svetlo-zlta
  WRITE(20,*) 'Arrows:  - moving displayed part in whole TF plane'
  WRITE(20,*) '---------------------------------------------------'
! -------------------------------
  RES = SETTEXTCOLORRGB(#0091C8FF)                 ! svetlo-oranzova
  WRITE(20,'(A\)') ' UP'
  RES = SETTEXTCOLORRGB(#00C0C0C0)                 ! biela (tlmena)
  WRITE(20,*) '..... up          (freq. direction)'
! --------------------------------
  RES = SETTEXTCOLORRGB(#0091C8FF)                 ! svetlo-oranzova
  WRITE(20,'(A\)') ' DOWN'
  RES = SETTEXTCOLORRGB(#00C0C0C0)                 ! biela (tlmena)
  WRITE(20,*) '... down        (freq. direction) '
! --------------------------------
  RES = SETTEXTCOLORRGB(#0091C8FF)                 ! svetlo-oranzova
  WRITE(20,'(A\)') ' <-'
  RES = SETTEXTCOLORRGB(#00C0C0C0)                 ! biela (tlmena)
  WRITE(20,*) '..... to the left  (time direction)'
! --------------------------------
  RES = SETTEXTCOLORRGB(#0091C8FF)                 ! svetlo-oranzova
  WRITE(20,'(A\)') ' ->'
  RES = SETTEXTCOLORRGB(#00C0C0C0)                 ! biela (tlmena)
  WRITE(20,*) '..... to the right (time direction)'

!----------------------------------------------------------------------
                              ! TF diagram window
  OPEN ( 55, FILE='user')

  WINFO.H        =  800*MAG
  WINFO.W        = 1100*MAG
  WINFO.X        = 0
  WINFO.Y        = 0
  WINFO.TYPE     = QWIN$SET
  RES = SETWSIZEQQ (55,WINFO)
  RES = SETACTIVEQQ(55)
  RES = PASSDIRKEYSQQ(.TRUE.)

  wc.numxpixels  = 1100*MAG
  wc.numypixels  =  800*MAG
  wc.numtextcols = -1
  wc.numtextrows = -1
  wc.numcolors   = -1
  wc.title       = "T-F Diagram"C
  wc.fontsize    = #0006000C
  RES = SETWINDOWCONFIG(WC)
  IF ( .NOT.(RES) ) RES = SETWINDOWCONFIG(WC)

!------- test hranic zvoleneho vyrezu

  CALL TEST_BORDERS ( 'T', TMIN, TMAX, TB, TE, KEY_LOG )
  CALL TEST_BORDERS ( 'F', FMIN, FMAX, FB, FE, KEY_LOG )


!----------- nacitanie analyzovaneho signalu

  READ ( 18 ) DAT(1:N)

!----------- nacitanie hodnot TFA
  DO I = 1, OFMAX
    READ ( 18 ) U  (1:N+1,I)
  END DO
    
  GLMAX = MAXVAL (U(1:N,:))             ! global. maximum v TF rovine
  IF ( KEY_AMPLOG ) THEN
    DO I = 1, OFMAX
      DO J = 1, N
        IF ( U(J,I) < GLMAX/100000. ) THEN
          U(J,I) = GLMAX/100000.
        END IF
        U(J,I) = LOG10(U(J,I)) - LOG10(GLMAX/100000.)
      END DO
    END DO
  ENDIF
  GLMAX = MAXVAL (U(1:N,:))             ! global. maximum v TF rovine

  IF (KEY_SQ) GLMAX=GLMAX*GLMAX

!------automat. urcenie delta medzi tikmi a formatu cislic pri osi
  IF (.NOT.KEY_LOG) THEN
    IF ( DFTICS < 0.) THEN
      DFTICS   = (FMAX-FMIN)/10. 
    ENDIF
    CALL AXIS ( FMAX, DFTICS, NDDF, NDF )
    IF ( (FMIN/DFTICS < 1.).AND.(FMIN/DFTICS /= 0.) ) THEN
      FMINTIC = (1. + INT(FMIN/DFTICS))*DFTICS
    ELSE
      FMINTIC =       INT(FMIN/DFTICS) *DFTICS
    ENDIF
  ENDIF
  IF ( DTTICS < 0.) THEN
    DTTICS   = (TMAX-TMIN)/10. 
  ENDIF
  CALL AXIS ( TMAX, DTTICS, NDDT, NDT ) 
  IF (TMIN < 0.) NDT=NDT+1
  IF ( (ABS(TMIN/DTTICS) < 1.).AND.(TMIN/DTTICS /= 0.) ) THEN
    TMINTIC = (1. + INT(TMIN/DTTICS))*DTTICS
  ELSE
    TMINTIC =       INT(TMIN/DTTICS) *DTTICS
  ENDIF
!----------- priprava dalsich vstupnych parametrov pre subr. TSFVIEW

  DO

    TICS_F = .FALSE.
    TICS_T = .FALSE.

    MTMIN = NINT((TMIN-TB)/DT  ) + 1
    MTMAX = NINT((TMAX-TB)/DT  ) + 1

    IF ( MTMAX - MTMIN + 1 > N ) THEN
      MTMAX = MTMAX -1
    END IF

    IF ( KEY_LOG ) THEN
      FMINTIC = 10**(AINT(LOG10(FMIN*1000000.))-7.)
      MFMIN   = NINT(LOG10(FMIN/FB)*MFHZ) + 1
      MFMAX   = NINT(LOG10(FMAX/FB)*MFHZ) + 1
    ELSE
      MFMIN   = NINT(     (FMIN-FB)*MFHZ) + 1
      MFMAX   = NINT(     (FMAX-FB)*MFHZ) + 1
    END IF

    IF ( TMINTIC < TMIN ) THEN
      TMINTIC  = TMIN
    END IF
    IF ( .NOT.KEY_LOG ) THEN
      IF ( FMINTIC < FMIN ) THEN
        FMINTIC  = FMIN
      END IF
    END IF

! -----------------VYPIS AKTUALNYCH NASTAVENI PRE VYKRESLOVANIE----------

    CALL SETTINGS (COMMENT, KEY_VIEW, KEY_INTER, KEY_MAX, MAG,             &
                   KEY_GRID, KEY_SAVE, DES_FILE, MAXSC, GLMAX,         &
                   NDT, NDDT, TMINTIC, DTTICS, TMIN, TMAX,             &
                   NDF, NDDF, FMINTIC, DFTICS, FMIN, FMAX, KEY_LOG )

!--------------------------------VYKRESLOVANIE--------------------------
    IF ( KEY_VIEW ) THEN
      CALL TSFVIEW ( U, DAT, KEY_SQ, KEY_INTER, FMIN, FMAX, TMIN, TMAX,&
                     MTMIN, MTMAX, MFMIN, MFMAX,                       &
                     MAG, MAXSC, KEY_LOG, KEY_GRID, KEY_MAX, WW,       &
                     KEY_SUMT,                                         &
                     DES_FILE, NUM_COLS, CG, SAVEBMP,                  &
                     TMINTIC, DTTICS, FMINTIC, DFTICS,                 &
                     NDT, NDDT, NDF, NDDF )
    END IF

! -------------------INTERAKTIVNE ZMENY PARAMETROV VYKRESLOVANIA---------
    TMIN1 = TMIN
    TMAX1 = TMAX
    FMIN1 = FMIN
    FMAX1 = FMAX

    RES = SETACTIVEQQ(0)
    RES = FOCUSQQ(0)
    CALL SETTEXTPOSITION (INT2(1), INT2(1), RC)
    RES = SETTEXTCOLORRGB(#00C8FFFF)                 ! svetlo-zlta
    CALL OUTTEXT('Press proper key or arrow from MAIN MENU. ')
    CALL SETTEXTPOSITION (INT2(2), INT2(1), RC)
    CALL OUTTEXT('-----------------------------------------')    
    CALL SETTEXTPOSITION (INT2(3), INT2(1), RC)
    RES = SETTEXTCOLORRGB(#0091C8FF)                 ! svetlo-oranzova      
    CALL OUTTEXT('  v   t   f   g   s   m   r   z   i   c                         q')    
    RES = SETTEXTCOLORRGB(#00C0C0C0)                 ! biela (tlmena)
    CALL OUTTEXT(' ...  for quit program')    
    RES = FOCUSQQ(55)
    KEY = GETCHARQQ()
! ---------------Test riadiacich klaves---------------- 
    IF      ( ICHAR(KEY) == #00 ) THEN  
      KEY = GETCHARQQ()
! F1 = 59, ..., F10 = 68
! -------------------Test sipiek-----------------------
    ELSE IF ( ICHAR(KEY) == #E0 ) THEN

      KEY = GETCHARQQ()
      IF      ( ICHAR(KEY) == 75 ) THEN  ! LEFT
        TMIN1 = 2.*TMIN - TMAX
        TMAX1 =    TMIN
      ELSE IF ( ICHAR(KEY) == 77 ) THEN  ! RIGHT
        TMIN1 = TMAX
        TMAX1 = 2.*TMAX - TMIN
      ELSE IF ( ICHAR(KEY) == 80 ) THEN  ! DOWN
        IF (KEY_LOG) THEN
          FMIN1 = 10 ** (2.* LOG10(FMIN)-LOG10(FMAX))
          FMAX1 = FMIN
        ELSE
          FMIN1 =        2.*       FMIN -      FMAX
          FMAX1 = FMIN
        END IF
      ELSE IF ( ICHAR(KEY) == 72 ) THEN  ! UP
        IF (KEY_LOG) THEN
          FMIN1 = FMAX
          FMAX1 = 10 ** (2.* LOG10(FMAX)-LOG10(FMIN))
        ELSE
          FMIN1 = FMAX
          FMAX1 =        2.*       FMAX -      FMIN
        END IF
      END IF

!------------------ Test pismen----------------------
    ELSE IF ( KEY == 'v' ) THEN            ! vypn./zapn. zobrazovania TFA roviny: (v)iew .............
      KEY_VIEW = .NOT.(KEY_VIEW)
      RES = FOCUSQQ(55)
    ELSE IF ( KEY == 'i' ) THEN            ! vypn./zapn. interpolovania TFA roviny: (i)nterpolation...
      KEY_INTER = .NOT.(KEY_INTER)
      RES = FOCUSQQ(55)
    ELSE IF ( KEY == 'r' ) THEN            ! vypn./zapn. zobrazovania (r)idges .............
      KEY_MAX = .NOT.(KEY_MAX)
      RES = FOCUSQQ(55)
    ELSE IF ( KEY == 'm' ) THEN            ! set magnification  of the TFA image...
      RES = SETACTIVEQQ(0)
      RES = FOCUSQQ(0)
      CALL CLEARSCREEN($GWINDOW)
      WRITE (OUTCH,'(A,F4.2, A)') 'Set new magnification of the TFA image       [',MAG,']:' 
      CALL OUTTEXT(OUTCH)
      CALL SETTEXTPOSITION (INT2(3), INT2(1), RC)
      WRITE (OUTCH,'(A)') 'HELP:  best window fit - 0.64'
      CALL OUTTEXT(OUTCH)
      CALL SETTEXTPOSITION (INT2(1), INT2(55), RC)
      LINCH = GETSTRQQ (INCH)
      IF ( LEN_TRIM(INCH) /= 0 )  READ (INCH,*) MAG         
      OPEN ( 99, FILE  ='SCALE.BMP')
      CLOSE( 99, STATUS='DELETE'   )
      CALL CLEARSCREEN($GWINDOW)
      RES = FOCUSQQ(55)
    ELSE IF ( KEY == 'g' ) THEN            ! parametre (g)ridu :    ..................................
      RES = SETACTIVEQQ(0)
      RES = FOCUSQQ(0)
      CALL CLEARSCREEN($GWINDOW)
      CALL OUTTEXT('Press: (d)raw grid   or  (c)olor of grid')
      KEY1 = GETCHARQQ()
      IF      ( KEY1 == 'd' ) THEN         !                      -(d)raw  .... vyp./zap.
        KEY_GRID = .NOT.(KEY_GRID)
      ELSE IF ( KEY1 == 'c' ) THEN         !                      -(c)olor
        DO
          CALL SETTEXTPOSITION (INT2(1), INT2(1), RC)
          WRITE (OUTCH,'(A,F4.2,A)') 'Grid color from <0.,1.>       [',CG,']  :'
          CALL OUTTEXT(OUTCH)
          CALL SETTEXTPOSITION (INT2(1), INT2(42), RC)
          LINCH = GETSTRQQ (INCH)
          IF ( LEN_TRIM(INCH) /= 0 ) READ (INCH,*) CG   
          IF ( ( CG >= 0. ) .AND. ( CG <= 1. ) ) EXIT ! [ak iba: <enter>]
        END DO
      END IF
      CALL CLEARSCREEN($GWINDOW)
      RES = FOCUSQQ(55)
    ELSE IF ( KEY == 'c' ) THEN            ! komentar :    ..................................
      RES = SETACTIVEQQ(0)
      RES = FOCUSQQ(0)
      CALL CLEARSCREEN($GWINDOW)
      CALL OUTTEXT('Press: (n)ew comment   or  (a)ppend to existing comment')
      KEY1 = GETCHARQQ()
      IF      ( KEY1 == 'a' ) THEN         !                      -(a)ppend 
        CALL CLEARSCREEN($GWINDOW)
        CALL SETTEXTPOSITION (INT2(1), INT2(1), RC)
        CC = 80 - LEN_TRIM(COMMENT)-1 
        WRITE (OUTCH,'(A,I2,A)') 'Type text to append to the existing comment (max.',CC,' char.):' 
        CALL OUTTEXT(OUTCH)
        CALL SETTEXTPOSITION (INT2(3), INT2(1), RC)
        LINCH = GETSTRQQ (OUTCH1)
        IF ( LEN_TRIM(OUTCH1) /= 0 ) COMMENT = TRIM(COMMENT)//' '//OUTCH1
      ELSE IF ( KEY1 == 'n' ) THEN         !                      -(n)ew comment

        CALL CLEARSCREEN($GWINDOW)
        CALL SETTEXTPOSITION (INT2(1), INT2(1), RC)
        CALL OUTTEXT('Type comment to the displayed picture (max. 80 char.):')
        CALL SETTEXTPOSITION (INT2(3), INT2(1), RC)
        LINCH = GETSTRQQ (OUTCH1)
        IF ( LEN_TRIM(OUTCH1) /= 0 ) COMMENT = OUTCH1
      ENDIF
      CALL CLEARSCREEN($GWINDOW)
      RES = FOCUSQQ(55)
    ELSE IF ( KEY == 's' ) THEN            ! parametre (s)cale :    ...................................
      RES = SETACTIVEQQ(0)
      RES = FOCUSQQ(0)
      CALL CLEARSCREEN($GWINDOW)
      CALL SETTEXTPOSITION (INT2(1), INT2(1), RC)
      CALL OUTTEXT('Press: (m)ax. value on scale  or  (f)ile with scale description')
      KEY1 = GETCHARQQ()
      IF      ( KEY1 == 'm' ) THEN         !                     -(m)ax. value on scale
        RES = SETACTIVEQQ(0)
        RES = FOCUSQQ(0)
        CALL SETTEXTPOSITION (INT2(1), INT2(1), RC)
        IF ( MAXSC < 0.) THEN
          WRITE (OUTCH,'(A)') 'Value for max. on scale            [ l ]  :'
        ELSE IF (ABS((MAXSC-GLMAX)/MAXSC) < 0.01) THEN
          WRITE (OUTCH,'(A)') 'Value for max. on scale            [ g ]  :'
        ELSE
          WRITE (OUTCH,'(A,E8.2, A)') 'Value for max. on scale       [',MAXSC,']  :' 
        END IF
        CALL OUTTEXT(OUTCH)
        CALL SETTEXTPOSITION (INT2(2), INT2(1), RC)
        WRITE (OUTCH,'(A)') 'HELP:  l - scaled for displayed part;   g - scaled for whole TF plane'
        CALL OUTTEXT(OUTCH)
        CALL SETTEXTPOSITION (INT2(3), INT2(1), RC)
        WRITE (OUTCH,'(A)') '       or you can input the number - new value in form: 2.11e-02'
        CALL OUTTEXT(OUTCH)
        CALL SETTEXTPOSITION (INT2(1), INT2(45), RC)
        LINCH = GETSTRQQ (INCH)
        IF ( LEN_TRIM(INCH) /= 0 ) THEN          
          IF      ( INCH == 'l') THEN
            MAXSC = -1.
          ELSE IF ( INCH == 'g') THEN
            MAXSC = GLMAX
          ELSE
            READ (INCH,*) MAXSC 
          END IF     
        END IF
      ELSE IF ( KEY1 == 'f' ) THEN         !                     -(f)ile with scale description
        RES = SETACTIVEQQ(0)
        RES = FOCUSQQ(0)
        DO
          CALL SETTEXTPOSITION (INT2(1), INT2(1), RC)
          WRITE (OUTCH,'(A,A12,A)') 'File with scale description (max.12 char.)   [',DES_FILE,']  :'
          CALL OUTTEXT(OUTCH)
          CALL SETTEXTPOSITION (INT2(1), INT2(67), RC)
          LINCH = GETSTRQQ (INCH)
          IF ( LEN_TRIM(INCH) /= 0 ) THEN
            INQUIRE (FILE = INCH, EXIST = EXISTS)
            IF (.NOT. EXISTS) THEN
              CALL SETTEXTPOSITION (INT2(3), INT2(1), RC)
              WRITE (OUTCH,'(A,A12,A)') 'ERROR: cannot find file: "', INCH,'" in working directory.'
              RES = SETTEXTCOLORRGB(#004646FF)           ! odtien cervenej
              CALL OUTTEXT(OUTCH)
              RES = SETTEXTCOLORRGB(#00C0C0C0)           ! biela (tlmena)
            ELSE
              DES_FILE = INCH
              OPEN ( 99, FILE  ='SCALE.BMP')
              CLOSE( 99, STATUS='DELETE'   )
              OPEN ( 10, FILE = DES_FILE )
              READ ( 10,* ) NUM_COLS
              CLOSE( 10 )
              EXIT  
            END IF
          ELSE     ! [ak iba: <enter>, nic nemeni]
            EXIT   
          ENDIF 
        END DO
      END IF
      CALL CLEARSCREEN($GWINDOW)
      RES = FOCUSQQ(55)
    ELSE IF ( KEY == 'z' ) THEN            ! zoom    ...........................................
      RES = SETACTIVEQQ(0)
      RES = FOCUSQQ(0)
      CALL CLEARSCREEN($GWINDOW)
      CALL SETTEXTPOSITION (INT2(1), INT2(1), RC)
      WRITE (OUTCH,'(A)') 'Press: (f)requency zoom   or  (t)ime zoom'
      CALL OUTTEXT(OUTCH)
      KEY1 = GETCHARQQ()
      IF      ( KEY1 == 'f' ) THEN         !                    -(f)requency zoom
        RES = SETACTIVEQQ(0)
        RES = FOCUSQQ(0)
        CALL CLEARSCREEN($GWINDOW)
        CALL SETTEXTPOSITION (INT2(1), INT2(1), RC)
        WRITE (OUTCH,'(A)') 'Press:  zoom (i)n  or  zoom (o)ut'
        CALL OUTTEXT(OUTCH)
        KEY2 = GETCHARQQ()
          IF      ( KEY2 == 'i' ) THEN     !                          - f. zoom in
            FMIN1 = FMIN
            FMAX1 = FMIN + 0.5*(FMAX -FMIN)
            TICS_F = .TRUE.
          ELSE IF   ( KEY2 == 'o' ) THEN     !                          - f. zoom out
            FMIN1 = FMIN
            FMAX1 = FMIN +   2*(FMAX -FMIN)
            TICS_F = .TRUE.
          ENDIF
      ELSE IF ( KEY1 == 't' ) THEN         !                    -(t)ime zoom
        RES = SETACTIVEQQ(0)
        RES = FOCUSQQ(0)
        CALL CLEARSCREEN($GWINDOW)
        CALL SETTEXTPOSITION (INT2(1), INT2(1), RC)
        WRITE (OUTCH,'(A)') 'Press:  zoom (i)n  or  zoom (o)ut'
        CALL OUTTEXT(OUTCH)
        KEY2 = GETCHARQQ()
          IF      ( KEY2 == 'i' ) THEN     !                          - t. zoom in
            TMIN1 = TMIN
            TMAX1 = TMIN + 0.5*(TMAX -TMIN)
            TICS_T = .TRUE.
          ELSE IF   ( KEY2 == 'o' ) THEN     !                          - t. zoom out
            TMIN1 = TMIN
            TMAX1 = TMIN +   2*(TMAX -TMIN)
            TICS_T = .TRUE.
          ENDIF
      END IF
      CALL CLEARSCREEN($GWINDOW)
      RES = FOCUSQQ(55)
    ELSE IF ( KEY == 't' ) THEN            ! time parameters...........................................
      RES = SETACTIVEQQ(0)
      RES = FOCUSQQ(0)
      CALL CLEARSCREEN($GWINDOW)
      CALL SETTEXTPOSITION (INT2(1), INT2(1), RC)
      WRITE (OUTCH,'(A)') 'Press: time (b)orders   or  time (a)xis'
      CALL OUTTEXT(OUTCH)
      KEY1 = GETCHARQQ()
      IF      ( KEY1 == 'b' ) THEN         !                    -(b)orders
        RES = SETACTIVEQQ(0)
        RES = FOCUSQQ(0)
        CALL CLEARSCREEN($GWINDOW)
        CALL SETTEXTPOSITION (INT2(1), INT2(1), RC)
        WRITE (OUTCH,'(A,F8.3, A)') 'First time to display       [',TMIN,'] s :' 
        CALL OUTTEXT(OUTCH)
        LINCH = GETSTRQQ (INCH)
        IF ( LEN_TRIM(INCH) /= 0 ) THEN
          READ (INCH,*) TMIN1
          TICS_T = .TRUE.
        ENDIF           

        CALL SETTEXTPOSITION (INT2(2), INT2(1), RC)
        WRITE (OUTCH,'(A,F8.3, A)') 'Last time to display        [',TMAX,'] s :' 
        CALL OUTTEXT(OUTCH)
        LINCH = GETSTRQQ (INCH)
        IF ( LEN_TRIM(INCH) /= 0 ) THEN  
          READ (INCH,*) TMAX1         
          TICS_T = .TRUE.
        ENDIF           
      ELSE IF ( KEY1 == 'a' ) THEN         !                    -(a)xis
        RES = SETACTIVEQQ(0)
        RES = FOCUSQQ(0)
        CALL CLEARSCREEN($GWINDOW)
        CALL SETTEXTPOSITION (INT2(1), INT2(1), RC)
        WRITE (OUTCH,'(A)') 'Press:   time (t)ics  or  (d)igits'
        CALL OUTTEXT(OUTCH)
        KEY2 = GETCHARQQ()
          IF      ( KEY2 == 't' ) THEN     !                          - tics
            CALL CLEARSCREEN($GWINDOW)
            CALL SETTEXTPOSITION (INT2(1), INT2(1), RC)
            WRITE (OUTCH,'(A,F9.3, A)') 'Draw the first tic at the time       [',TMINTIC,'] s :' 
            CALL OUTTEXT(OUTCH)
            LINCH = GETSTRQQ (INCH)
            IF ( LEN_TRIM(INCH) /= 0 )  READ (INCH,*) TMINTIC         

            CALL SETTEXTPOSITION (INT2(2), INT2(1), RC)
            WRITE (OUTCH,'(A,F8.3, A)') 'Time interval between tics           [',DTTICS,' ] s :' 
            CALL OUTTEXT(OUTCH)
            LINCH = GETSTRQQ (INCH)
            IF ( LEN_TRIM(INCH) /= 0 )  READ (INCH,*) DTTICS         
          ELSE IF   ( KEY2 == 'd' ) THEN     !                          - digits
            CALL CLEARSCREEN($GWINDOW)
            CALL SETTEXTPOSITION (INT2(1), INT2(1), RC)
            WRITE (OUTCH,'(A,I8, A)') 'Number of all digits ( including . )       [',NDT,']  :' 
            CALL OUTTEXT(OUTCH)
            LINCH = GETSTRQQ (INCH)
            IF ( LEN_TRIM(INCH) /= 0 )  READ (INCH,*) NDT         

            CALL SETTEXTPOSITION (INT2(2), INT2(1), RC)
            WRITE (OUTCH,'(A,I2, A)') 'Number of decimal digits                        [',NDDT,']  :' 
            CALL OUTTEXT(OUTCH)
            LINCH = GETSTRQQ (INCH)
            IF ( LEN_TRIM(INCH) /= 0 )  READ (INCH,*) NDDT         
          ENDIF
      END IF
      CALL CLEARSCREEN($GWINDOW)
      RES = FOCUSQQ(55)
    ELSE IF ( KEY == 'f' ) THEN            ! frequency parameters.....................................
      RES = SETACTIVEQQ(0)
      RES = FOCUSQQ(0)
      CALL CLEARSCREEN($GWINDOW)
      CALL SETTEXTPOSITION (INT2(1), INT2(1), RC)
      IF (.NOT. KEY_LOG) THEN
        WRITE (OUTCH,'(A)') 'Press:   frequency (b)orders   or  frequency (a)xis'
        CALL OUTTEXT(OUTCH)
        KEY1 = GETCHARQQ()
        IF      ( KEY1 == 'b' ) THEN         !                    -(b)orders
          RES = SETACTIVEQQ(0)
          RES = FOCUSQQ(0)
          CALL CLEARSCREEN($GWINDOW)
          CALL SETTEXTPOSITION (INT2(1), INT2(1), RC)
          WRITE (OUTCH,'(A,F8.3, A)') 'First frequency to display       [',FMIN,'] Hz :' 
          CALL OUTTEXT(OUTCH)
          LINCH = GETSTRQQ (INCH)
          IF ( LEN_TRIM(INCH) /= 0 )  THEN
            READ (INCH,*) FMIN1         
            TICS_F = .TRUE.
          ENDIF           

          CALL SETTEXTPOSITION (INT2(2), INT2(1), RC)
          WRITE (OUTCH,'(A,F8.3, A)') 'Last frequency to display        [',FMAX,'] Hz :' 
          CALL OUTTEXT(OUTCH)
          LINCH = GETSTRQQ (INCH)
          IF ( LEN_TRIM(INCH) /= 0 )  THEN
            READ (INCH,*) FMAX1         
            TICS_F = .TRUE.
          ENDIF           
        ELSE IF ( KEY1 == 'a' ) THEN         !                    -(a)xis
          RES = SETACTIVEQQ(0)
          RES = FOCUSQQ(0)
          CALL CLEARSCREEN($GWINDOW)
          CALL SETTEXTPOSITION (INT2(1), INT2(1), RC)
          WRITE (OUTCH,'(A)') 'Press:   frequency (t)ics  or  (d)igits'
          CALL OUTTEXT(OUTCH)
          KEY2 = GETCHARQQ()
            IF      ( KEY2 == 't' ) THEN     !                          - tics
              CALL CLEARSCREEN($GWINDOW)
              CALL SETTEXTPOSITION (INT2(1), INT2(1), RC)
              WRITE (OUTCH,'(A,F9.3, A)') 'Draw the first tic at the frequency       [',FMINTIC,'] Hz :' 
              CALL OUTTEXT(OUTCH)
              LINCH = GETSTRQQ (INCH)
              IF ( LEN_TRIM(INCH) /= 0 )  READ (INCH,*) FMINTIC         

              CALL SETTEXTPOSITION (INT2(2), INT2(1), RC)
              WRITE (OUTCH,'(A,F8.3, A)') 'Frequency interval between tics           [',DFTICS,' ] Hz :' 
              CALL OUTTEXT(OUTCH)
              LINCH = GETSTRQQ (INCH)
              IF ( LEN_TRIM(INCH) /= 0 )  READ (INCH,*) DFTICS         
            ELSE IF   ( KEY2 == 'd' ) THEN     !                          - digits
              CALL CLEARSCREEN($GWINDOW)
              CALL SETTEXTPOSITION (INT2(1), INT2(1), RC)
              WRITE (OUTCH,'(A,I8, A)') 'Number of all digits ( including . )       [',NDF,']  :' 
              CALL OUTTEXT(OUTCH)
              LINCH = GETSTRQQ (INCH)
              IF ( LEN_TRIM(INCH) /= 0 )  READ (INCH,*) NDF         

              CALL SETTEXTPOSITION (INT2(2), INT2(1), RC)
              WRITE (OUTCH,'(A,I2, A)') 'Number of decimal digits                       [',NDDF,']  :' 
              CALL OUTTEXT(OUTCH)
              LINCH = GETSTRQQ (INCH)
              IF ( LEN_TRIM(INCH) /= 0 )  READ (INCH,*) NDDF         
            ENDIF
        END IF
      ELSE
        WRITE (OUTCH,'(A,F8.3, A)') 'First frequency to display       [',FMIN,'] Hz :' 
        CALL OUTTEXT(OUTCH)
        LINCH = GETSTRQQ (INCH)
        IF ( LEN_TRIM(INCH) /= 0 )  READ (INCH,*) FMIN1         

        CALL SETTEXTPOSITION (INT2(2), INT2(1), RC)
        WRITE (OUTCH,'(A,F8.3, A)') 'Last frequency to display        [',FMAX,'] Hz :' 
        CALL OUTTEXT(OUTCH)
        LINCH = GETSTRQQ (INCH)
        IF ( LEN_TRIM(INCH) /= 0 )  READ (INCH,*) FMAX1         
      ENDIF
      CALL CLEARSCREEN($GWINDOW)
      RES = FOCUSQQ(55)
    ELSE IF ( KEY == 'q' ) THEN             ! ukoncenie programu ................................
      OPEN ( 99, FILE  ='SCALE.BMP')
      CLOSE( 99, STATUS='DELETE'   )
      OPEN ( 99, FILE  ='TF.BMP'   )
      CLOSE( 99, STATUS='DELETE'   )
      EXIT
    END IF

    CALL TEST_BORDERS ( 'T', TMIN1, TMAX1, TB, TE, KEY_LOG )
    TMINTIC = TMINTIC + ( TMIN1 - TMIN )
    TMIN    = TMIN1
    TMAX    = TMAX1
    IF (TICS_T) THEN
      DTTICS   = (TMAX-TMIN)/10. 
      CALL AXIS ( TMAX, DTTICS, NDDT, NDT ) 
      IF (  (ABS(TMIN/DTTICS) < 1.).AND.(TMIN/DTTICS /= 0.)) THEN
        IF (TMIN < 0.) THEN
          TMINTIC =       INT(TMIN/DTTICS)*DTTICS
        ELSE
          TMINTIC = (1. + INT(TMIN/DTTICS))*DTTICS
        ENDIF
      ELSE
        TMINTIC =       INT(TMIN/DTTICS) *DTTICS
      ENDIF
      IF (TMIN < 0.) NDT=NDT+1
    ENDIF
    CALL TEST_BORDERS ( 'F', FMIN1, FMAX1, FB, FE, KEY_LOG )
    IF (.NOT.KEY_LOG) FMINTIC = FMINTIC + ( FMIN1 - FMIN )
    FMIN    = FMIN1
    FMAX    = FMAX1
    IF ( (.NOT.KEY_LOG).AND.(TICS_F) ) THEN
      DFTICS   = (FMAX-FMIN)/10. 
      CALL AXIS ( FMAX, DFTICS, NDDF, NDF )
      IF ( (FMIN/DFTICS < 1.).AND.(FMIN/DFTICS /= 0.) ) THEN
        FMINTIC = (1. + INT(FMIN/DFTICS))*DFTICS
      ELSE
        FMINTIC =       INT(FMIN/DFTICS) *DFTICS
      ENDIF
    ENDIF
  END DO

  CLOSE (18)

END PROGRAM
